@@ -91,7 +91,7 @@ def create_attribute_string(attribute, attribute_class, name, value, is_string):
     # type: (str, str, str, typing.Any, bool) -> str
     if is_string:
         value = '"' + value + '"'
-    elif not value:
+    elif value is None:
         value = '""'
 
     attribute_string = 'BA_ "' + attribute + '" ' + attribute_class + ' ' + name + ' ' + str(value) + ';\n'
@@ -200,16 +200,20 @@ def dump(in_db, f, **options):
     output_names = collections.defaultdict(dict)  # type: typing.Dict[canmatrix.Frame, typing.Dict[canmatrix.Signal, str]]
 
     for frame in db.frames:
-        # fix long frame names
+        # fix long frame names , warn if the frame name exceeds 32 characters
         if len(frame.name) > 32:
             frame.add_attribute("SystemMessageLongSymbol", frame.name)
+            logger.warning("Frame %s name exceeds 32 characters, consider updating the frame name within"
+                           " character limit(Max 32 characters) ", frame.name)
             frame.name = frame.name[0:32]
             db.add_frame_defines("SystemMessageLongSymbol", "STRING")
 
-        # fix long signal names
+        # fix long signal names, warn if the signal name exceeds 32 characters
         for s in frame.signals:
             if len(s.name) > 32:
                 s.add_attribute("SystemSignalLongSymbol", s.name)
+                logger.warning("Signal %s::%s name exceeds 32 characters, consider updating the signal name "
+                               "within the character limit(Max 32 characters)", frame.name, s.name)
                 s.name = s.name[0:32]
                 db.add_signal_defines("SystemSignalLongSymbol", "STRING")
 
@@ -233,8 +237,13 @@ def dump(in_db, f, **options):
         for signal in frame.signals:
             if signal.cycle_time != 0:
                 signal.add_attribute("GenSigCycleTime", signal.cycle_time)
-            if signal.phys2raw(None) != 0:
-                signal.add_attribute("GenSigStartValue", signal.phys2raw(None))
+            if "GenSigStartValue" in db.signal_defines:
+                if signal.phys2raw(None) != 0:
+                    if db.signal_defines["GenSigStartValue"].defaultValue is not None and \
+                            float(signal.initial_value) != float(db.signal_defines["GenSigStartValue"].defaultValue):
+                        signal.add_attribute("GenSigStartValue", signal.phys2raw(float(db.signal_defines["GenSigStartValue"].defaultValue)))
+                    elif db.signal_defines["GenSigStartValue"].defaultValue is None:
+                        signal.add_attribute("GenSigStartValue", signal.phys2raw(None))
 
             name = normalized_names[signal]
             if compatibility:
@@ -414,13 +423,7 @@ def dump(in_db, f, **options):
 
     # signal-values:
     for frame in db.frames:
-        multiplex_written = False
         for signal in frame.signals:
-            if signal.multiplex == 'Multiplexor' and multiplex_written:
-                continue
-
-            multiplex_written = True
-
             if signal.values:
                 f.write(
                     ('VAL_ %d ' %
@@ -543,7 +546,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                 continue
             decoded = l.decode(dbc_import_encoding).strip()
             if decoded.startswith("BO_ "):
-                regexp = re.compile(r"^BO_ ([^\ ]+) ([^\ ]+) *: ([^\ ]+) ([^\ ]+)")
+                regexp = re.compile(r"^BO_ ([^\ ]+) ([^\ ]+) *: *([^\ ]+) ([^\ ]+)")
                 temp = regexp.match(decoded)
     #            db.frames.addFrame(Frame(temp.group(1), temp.group(2), temp.group(3), temp.group(4)))
                 frame = canmatrix.Frame(temp.group(2), arbitration_id=int(temp.group(1)),
@@ -952,7 +955,12 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
         #     frame.extended = 1
 
         for signal in frame.signals:
-            gen_sig_start_value = float_factory(signal.attributes.get("GenSigStartValue", "0"))
+            if "GenSigStartValue" in db.signal_defines \
+                    and db.signal_defines["GenSigStartValue"].defaultValue is not None:
+                default_value = signal.phys2raw(float_factory(db.signal_defines["GenSigStartValue"].defaultValue))
+            else:
+                default_value = signal.phys2raw(None)
+            gen_sig_start_value = float_factory(signal.attributes.get("GenSigStartValue", default_value))
             signal.initial_value = (gen_sig_start_value * signal.factor) + signal.offset
             signal.cycle_time = int(signal.attributes.get("GenSigCycleTime", 0))
             if signal.attribute("SystemSignalLongSymbol") is not None:
