@@ -5,6 +5,7 @@ import csv
 import shlex
 import sys
 import typing
+from string import hexdigits
 from builtins import *
 
 if sys.version_info >= (3, 5):
@@ -90,6 +91,12 @@ def guess_value(text_value):  # type: (str) -> str
         return "0"
     elif text_value in ["true", "on"]:
         return "1"
+    elif text_value[:2] == "0b":
+        if text_value[2:].isdecimal():
+            return str(int(text_value[2:], 2))
+    elif text_value[:2] == "0x":
+        if all([f in hexdigits for f in text_value[2:]]):
+            return str(int(text_value[2:], 16))
     return text_value
 
 
@@ -107,13 +114,15 @@ def get_gcd(value1, value2):  # type (int,int) -> (int)
     else:
         return fractions.gcd(value1, value2)
 
+
 def decode_number(value, float_factory):  # type(string) -> (int)
     """
     Decode string to integer and guess correct base
     :param value: string input value
     :return: integer
     """
-
+    if value is None:
+        return 0
     value = value.strip()
 
     if '.' in value:
@@ -127,4 +136,4 @@ def decode_number(value, float_factory):  # type(string) -> (int)
         base = 16
         value = value[2:]
 
-    return int(value, base)
\ No newline at end of file
+    return int(value, base)
